// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef CONFIG_HPP
#define CONFIG_HPP

#include <string>
#include <vector>
#include <utility>
#include <SDL.h>

// imposed minimum grid values
#define ROWS_MIN 3
#define COLS_MIN 4
#define COLS_MUL 1.33
#define ROWS_MAX 75
#define COLS_MAX 100

// audio volume
#define MAX_VOLUME 10

// maximum allowed game score
#define MAX_SCORE 9999999

// player limit
#define MAX_PLAYER_AMOUNT 4

class Config{
	public:
		// initializes the Config [depends on: -]
		static void init();
		// saves the present config
		static void save();

		// each player controls structure
		typedef struct {
			SDLKey kescape;
			SDLKey kimm;
			SDLKey kbomb;
			SDLKey kup;
			SDLKey kdown;
			SDLKey kright;
			SDLKey kleft;
			SDLKey kfireup;
			SDLKey kfiredown;
			SDLKey kfireright;
			SDLKey kfireleft;

			std::pair<int,int> jescape;
			std::pair<int,int> jimm;
			std::pair<int,int> jbomb;
			std::pair<int,int> jupdown;
			bool jinvertupdown;
			std::pair<int,int> jrightleft;
			bool jinvertrightleft;
			std::pair<int,int> jfireupdown;
			bool jinvertfireupdown;
			std::pair<int,int> jfirerightleft;
			bool jinvertfirerightleft;
		} playercontrols;

		// getters and setters for screen properties
		static int  get_resx();
		static void set_resx( int rx );
		static int  get_resy();
		static void set_resy( int ry );
		static int  get_bitdepth();
		static void set_bitdepth( int bd );
		static bool get_fullscreen();
		static void set_fullscreen( bool f );
		static bool get_linesmooth();
		static void set_linesmooth( bool ls );
		// getters and setters for grid properties
		static bool get_grid();
		static void set_grid( bool g );
		static int  get_gridrows();
		static void set_gridrows( int gr );
		static int  get_gridcols();
		static void set_gridcols( int gc );
		static bool get_gridbezier();
		static void set_gridbezier( bool gb );
		// getter and setter for debris
		static bool get_debris();
		static void set_debris( bool d );
		// getter and setter for audio
		static int  get_audio();
		static void set_audio( int a );
		// getter for hiscore
		static int get_hiscore();
		// getter and setter for language
		static const std::string& get_lang();
		static void set_lang( const std::string& l );
		// getter and setter for controls
		static const std::vector<playercontrols>& get_controls();
		static void set_controls( const std::vector<playercontrols>& c );

	private:
		// loads saved config, returns false if error while loading.
		// it does not erase present config, so stored config can just
		// have a couple of values to override part of the defaults
		static bool load();
		// loads default values
		static void defaults();

		// config file
		static std::string configfile;

		// screen properties
		static int  resx;
		static int  resy;
		static int  bitdepth;
		static bool fullscreen;
		static bool linesmooth;
		// grid properties
		static bool grid;
		static int  gridrows;
		static int  gridcols;
		static bool gridbezier;
		// debris
		static bool debris;
		// audio
		static int audio;
		// hiscore
		static int hiscore;
		// language
		static std::string lang;
		// controls
		static std::vector<playercontrols> controls;
};

#endif // Config.hpp
