// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include <clocale>
#include <iostream>
#include <fstream>
#include <utility>
#include "util/Text.hpp"
#include "Config.hpp"
#include "gui/GUI.hpp"
#include "gui/Menu.hpp"
#include "Main.hpp"
#include "Locale.hpp"

#if !defined(LOCALE_FILE_PREFIX)
# define LOCALE_FILE_PREFIX FILES_DIRECTORY
#endif // not defined LOCALE_FILE_PREFIX

// file name

#define LOCALE_NAME "locale"

// available languages
// always at least one: the present language
std::set<std::string> Locale::available;

// every single language data available
std::map< std::string, std::vector<std::string> > Locale::langs;

// preinitializes the locale [depends on: -]
void Locale::preinit(){
	std::setlocale( LC_ALL, "C" );
}
// initializes the locale [depends on: Config]
void Locale::init(){
	load();

	// if present language isn't known
	if ( ! langs.count( Config::get_lang() ) ){
		std::cerr << "FATAL: unknown language \"" << Config::get_lang() << "\"" << std::endl;
		Main::quit( Main::ERROR_LOCALE );
	}
}
// reinitializes the locale, call it when config changes
void Locale::reinit(){
	gui::GUI::initdrawing();
	gui::Menu::initdrawing();
}
// ends the locale
void Locale::finalize(){
	langs.clear();
	available.clear();
}
// gets the string corresponding to the code
const std::string& Locale::get( int code ){
	return (*langs.find(Config::get_lang())).second[code];
}
// gets the available languages
const std::set<std::string>& Locale::get_availablelangs(){
	return available;
}
// loads the locale data, best-effort: it loads as much as it can
void Locale::load(){
	std::ifstream in( LOCALE_FILE_PREFIX LOCALE_NAME, std::ifstream::in );
	if ( ! in ){
		std::cerr << "ERROR: cannot open the locale file \"" LOCALE_FILE_PREFIX LOCALE_NAME "\"" << std::endl;
		return;
	}

	while( in.good() ){
		// read a line
		std::string str;
		std::getline( in, str );
		if ( ! in )
			break;

		// getting language name
		std::string langname = util::Text::trim( util::Text::eraseNewLine( util::Text::utf8SkipBOM( str ) ) );

		// getting all language text
		std::vector<std::string> lines;
		while( (lines.size() < STR_AMOUNT) && in.good() ){
			std::getline( in, str );
			if ( ! in )
				break;

			str = util::Text::eraseNewLine( str );
			// removing first char (which should be a useless tab)
			if ( ! str.empty() )
				str.erase( 0, 1 );

			lines.push_back( str );
		}
		if ( lines.size() == STR_AMOUNT ){
			// saving the language
			langs.insert( std::pair< std::string, std::vector<std::string> >( langname, lines ) );
			available.insert( langname );
		}else
			std::cerr << "ERROR: cannot properly read the locale file \"" LOCALE_FILE_PREFIX LOCALE_NAME "\"" << std::endl;

		str.clear();
		lines.clear();
		langname.clear();
	}

	in.close();
}
