// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef ENEMYSCHEDULER_HPP
#define ENEMYSCHEDULER_HPP

#include <istream>
#include <ostream>
#include "../util/Real.hpp"
#include "../util/Vector.hpp"

namespace au {

class EnemyScheduler{
	public:
		// initializes the scheduler [depends on: Constants]
		static void init();
		// stops present spawning
		static void stop();
		// updates spawning
		static void update( int milliseconds );
		// updates the spawning probabilities
		static void update_prob();

		// serializes the enemy scheduler
		static void serialize( std::ostream& out );
		// unserializes the enemy scheduler
		static void unserialize( std::istream& in );

	private:
		// adds and normalizes the probabilities in [0,1], filling spawning_prob_added
		static void normalize();

		// fixed spawning points
		static util::Vector points[];

		// time till next spawn
		static int spawntime;

		// is there no spawning process active?
		static bool nospawning;

		// which enemies are spawning
		static bool states[];
		// time left for spawning for each enemy type
		static int spawningtimes[];
		// spawning points for each enemy type
		static int spawningpoints[];

		// spawning probability of each enemy
		static real spawning_prob[];
		// spawning probability of each enemy, added and normalized
		static real spawning_prob_added[];
		// the enemy is spawned just once at a time
		static bool spawning_single[];
		// time the spawning process lasts
		static int  spawning_time[];
		// time between consecutive spawns
		static int  spawning_interval[];
		// the enemy spawns from random locations
		static bool spawning_rand[];
};

}

#endif // EnemyScheduler.hpp
