// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef EXAMINIS_HPP
#define EXAMINIS_HPP

#include <set>
#include "Automaton.hpp"

namespace au {

// the class that controls the examinis enemies
class Examinis : public Automaton{
 	public:
		// ctor
		Examinis( const util::Vector& xy, int waited = 0, int reproduce = 0, bool create = true );
		// dtor
		virtual ~Examinis();

		// unserializes the automaton
		static void unserialize( std::istream& in );

		// remove the apicula whose dtor is being executed
		void takeout( ExaminisApicula *me );

	private:
		// computes the reproduce time
		int compute_reproduce() const;

		// main drone
		ExaminisApicula *queen;
		// apicula drones
		std::set<ExaminisApicula*> apiculas; // unordered

		// time already waited for next reproduction
		int waited;
		// time till a new drone is created
		int reproduce;

	// redefined automaton functions

	public:
		// serializes the automaton
		virtual void serialize( std::ostream& out );

	protected:
		// drawing function
		virtual void specific_draw();
		// where is the nearest player. itis is an unit vector or zero.
		// distance is the distance or REAL_MAX
		virtual void specific_there( const util::Vector& itis, real distance );
		// process function
		virtual void specific_process( int milliseconds );
};

}

#endif // Examinis.hpp
