// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include "../Main.hpp"
#include "ExaminisApicula.hpp"

// ctor
au::ExaminisApicula::ExaminisApicula( Examinis *swarmer, const util::Vector& xy, bool queen,
                                      bool play, real phase, real phase2, real orbit_ratiomax ) :
 Automaton( xy, util::Vector(0.0,0.0), queen ? rc::examinis_queen_r() : rc::examinis_apicula_r() ),
 queen( queen ),
 lifes( queen ? rc::examinis_queen_lifes() : rc::examinis_apicula_lifes() ),
 phase( phase ),
 phase2( phase2 ),
 orbit_ratiomax( orbit_ratiomax ),
 swarmer( swarmer )
{
	start_activecollider( EXAMINIS_COL );
	start_passivecollider( PLAYER_COL );
	start_enemy( play );
}
// dtor
au::ExaminisApicula::~ExaminisApicula(){
	if ( dolitter ){
		if ( queen ){
			Automaton::create_debris( xy, rc::examinis_queen_litter_r(),
			                          rc::examinis_queen_litter_c_r(), rc::examinis_queen_litter_c_g(), rc::examinis_queen_litter_c_b() );
		}else{
			Automaton::create_debris( xy, rc::examinis_apicula_litter_r(),
			                          rc::examinis_apicula_litter_c_r(), rc::examinis_apicula_litter_c_g(), rc::examinis_apicula_litter_c_b() );
		}
	}
	swarmer->takeout( this );
}
// gets current lifes
int au::ExaminisApicula::get_lifes() const{
	return lifes;
}
// gets current phase
real au::ExaminisApicula::get_phase() const{
	return phase;
}
// sets the phase
void au::ExaminisApicula::set_phase( real newphase ){
	phase = newphase;
}
// gets current phase2
real au::ExaminisApicula::get_phase2() const{
	return phase2;
}
// sets the phase2
void au::ExaminisApicula::set_phase2( real newphase2 ){
	phase2 = newphase2;
}
// gets orbit parameters
real au::ExaminisApicula::get_orbit_ratiomax() const{
	return orbit_ratiomax;
}
// collision checking
int au::ExaminisApicula::shot_collision( Automaton *p ){
	if ( queen ){
		--lifes;
		// adding points for a queen hit
		Main::add_score( get_hitscore() );
		// and maybe for a kill
		if ( ! lifes )
			Main::add_score( get_killscore() );
		// killing?
		return lifes ? PASSIVE_DIES : BOTH_DIE;
	}else{
		// a simple apicula reflects shots
		util::Vector v2;
		if ( reflect_collision( p, -1.0, v2 ) ){
			--lifes;
			p->bounce( v2 );
		}
		// killing?
		return lifes ? NO_DEATHS : ACTIVE_DIES;
	}
}
// each collision type for active colliders
void au::ExaminisApicula::recieve_collision( int& res, Bullet *p ){
	res = shot_collision( p );
}
void au::ExaminisApicula::recieve_collision( int& res, Laser *p ){
	res = shot_collision( p );
}
void au::ExaminisApicula::recieve_collision( int& res, LightningSpark *p ){
	res = shot_collision( p );
}
// serializes the automaton
void au::ExaminisApicula::serialize( std::ostream& out ){
	out << xy
	    << " " << lifes;
	if ( ! queen )
		out << " " << phase << " " << phase2 << " " << orbit_ratiomax;
}
// unserializes the automaton
au::ExaminisApicula *au::ExaminisApicula::unserialize( std::istream& in, Examinis *swarmer, bool queen ){
	util::Vector xy;
	int lifes;

	in >> xy
	   >> lifes;

	ExaminisApicula *res;
	if ( ! queen ){
		real phase;
		real phase2;
		real orbit_ratiomax;

		in >> phase >> phase2 >> orbit_ratiomax;

		if ( (lifes <= 0) || (lifes > rc::examinis_apicula_lifes()) )
			throw Main::UNSERIAL_EXAMINISAPICULA_LIFES;

		res = new ExaminisApicula( swarmer, xy, queen, false, phase, phase2, orbit_ratiomax );
	}else{
		if ( (lifes <= 0) || (lifes > rc::examinis_queen_lifes()) )
			throw Main::UNSERIAL_EXAMINISAPICULA_LIFES;

		res = new ExaminisApicula( swarmer, xy, queen, false );
	}
	res->lifes = lifes;

	return res;
}
// collision functions
void au::ExaminisApicula::send_collision( int& res, Automaton *a ){
	a->recieve_collision( res, this );
}
// scoring points
int au::ExaminisApicula::get_killscore() const{
	return rc::examinis_kill_score();
}
int au::ExaminisApicula::get_hitscore() const{
	return rc::examinis_hit_score();
}
