// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef EXAMINISAPICULA_HPP
#define EXAMINISAPICULA_HPP

#include "Automaton.hpp"
#include "../rc/Constants.hpp"
#include "../util/Random.hpp"

namespace au {

// the examinis apicula enemy
class ExaminisApicula : public Automaton{
 	public:
		// ctor
		ExaminisApicula( Examinis *swarmer, const util::Vector& xy, bool queen, bool play = true,
		                 real phase = PI/2.0, real phase2 = util::Random::get(0.0, 2.0*PI),
		                 real orbit_ratiomax = util::Random::get(rc::examinis_orbitmin_r(),rc::examinis_orbitmax_r())/rc::examinis_orbitmin_r() );
		// dtor
		virtual ~ExaminisApicula();

		// unserializes the automaton
		static ExaminisApicula *unserialize( std::istream& in, Examinis *swarmer, bool queen );

		// gets current lifes
		int get_lifes() const;

		// gets current phase
		real get_phase() const;
		// sets the phase
		void set_phase( real newphase );

		// gets current phase2
		real get_phase2() const;
		// sets the phase2
		void set_phase2( real newphase2 );

		// gets orbit parameters
		real get_orbit_ratiomax() const;

	private:
		// collision checking
		int shot_collision( Automaton *p );

		// is the queen?
		bool queen;
		// lifes
		int lifes;
		// phase angle in the ellipse
		real phase;
		// phase2 ellipse's rotation angle
		real phase2;
		// orbit parameters
		real orbit_ratiomax;

		// central brain
		Examinis *swarmer;

	// redefined automaton functions

	public:
		// each collision type for active colliders
		virtual void recieve_collision( int& res, Bullet *p );
		virtual void recieve_collision( int& res, Laser *p );
		virtual void recieve_collision( int& res, LightningSpark *p );
		// scoring points
		virtual int get_killscore() const;
		virtual int get_hitscore() const;
		// serializes the automaton
		virtual void serialize( std::ostream& out );

	protected:
		// collision functions
		virtual void send_collision( int& res, Automaton *a );
};

}

#endif // ExaminisApicula.hpp
