// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef LAGVARD_HPP
#define LAGVARD_HPP

#include "Automaton.hpp"

namespace au {

// the lagvard enemy
class Lagvard : public Automaton{
	public:
		// ctor
		Lagvard( const util::Vector& xy, int t = 0, bool play = true );
		// dtor
		virtual ~Lagvard();

		// unserializes the automaton
		static void unserialize( std::istream& in );

	// redefined automaton functions

	public:
		// scoring points
		virtual int get_killscore() const;
		// serializes the automaton
		virtual void serialize( std::ostream& out );

	protected:
		// drawing function
		virtual void specific_draw();
		// where is the nearest player. itis is an unit vector or zero.
		// distance is the distance or REAL_MAX
		virtual void specific_there( const util::Vector& itis, real distance );
		// process function
		virtual void specific_process( int milliseconds );
		// collision functions
		virtual void send_collision( int& res, Automaton *a );
};

}

#endif // Lagvard.hpp
