// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include "../rc/Constants.hpp"
#include "../vis/Draw.hpp"
#include "../Audio.hpp"
#include "../Main.hpp"
#include "LaserTail.hpp"
#include "Laser.hpp"

// ctor
au::Laser::Laser( const util::Vector& xy, const util::Vector& v, const util::Vector& v2, int lifes, bool play ) :
 Automaton( xy, v, rc::laser_r() ),
 lifes( lifes ),
 o( xy ),
 v2( v2 ),
 parts( new LaserParts( v.magnitude(), v2.magnitude() ) )
{
	start_activecollider( SHOT_COL );
	start_passivecollider( EXAMINIS_COL | REDHOLE_COL | SHIELDED_COL );
	start_processable();
	start_shot();
	start_visible( LASER_PRIO );

	if ( play )
		Audio::playSound( Audio::SOUND_LASER_FIRE );
}
// dtor
au::Laser::~Laser(){
	if ( dolitter )
		Automaton::create_debris( xy, rc::laser_litter_r(), rc::laser_litter_c_r(), rc::laser_litter_c_g(), rc::laser_litter_c_b() );

	// adding last part, the point where the laser was killed
	parts->pushpart( xy, v.angle() );

	Automaton::create_lasertail( o, v2, parts );
}
// bouncing function
void au::Laser::bounce( const util::Vector& newv ){
	v = newv;

	// a stop point is created when bouncing
	parts->pushpart( xy, newv.angle() );
}
// each collision type for active colliders
void au::Laser::recieve_collision( int& res, Lagvard *p ){
	Main::add_score( p->get_killscore() );
	res = ( --lifes ) ? PASSIVE_DIES : BOTH_DIE;
}
void au::Laser::recieve_collision( int& res, Proton *p ){
	Main::add_score( p->get_killscore() );
	res = ( --lifes ) ? PASSIVE_DIES : BOTH_DIE;
}
void au::Laser::recieve_collision( int& res, Scared *p ){
	Main::add_score( p->get_killscore() );
	res = ( --lifes ) ? PASSIVE_DIES : BOTH_DIE;
}
void au::Laser::recieve_collision( int& res, Spawned *p ){
	Main::add_score( p->get_killscore() );
	res = ( --lifes ) ? PASSIVE_DIES : BOTH_DIE;
}
void au::Laser::recieve_collision( int& res, Spawnship *p ){
	Main::add_score( p->get_killscore() );
	res = ( --lifes ) ? PASSIVE_DIES : BOTH_DIE;
}
// serializes the automaton
void au::Laser::serialize( std::ostream& out ){
	out << Automaton::LASER_SERIAL
	    << " " << xy << " " << v
	    << " " << lifes << " " << o << " " << v2 << " " << (*parts) << std::endl;
}
// unserializes the automaton
void au::Laser::unserialize( std::istream& in ){
	util::Vector xy;
	util::Vector v;
	int lifes;
	util::Vector o;
	util::Vector v2;

	in >> xy >> v
	   >> lifes >> o >> v2;
	if ( lifes <= 0 )
		throw Main::UNSERIAL_LASER_LIFES;

	Laser *l = new Laser( xy, v, v2, lifes, false );
	l->o = o;
	in >> (*(l->parts));
}
// drawing function
void au::Laser::specific_draw(){
	vis::Draw::start_line_strip();
		vis::Draw::set_color( rc::laser_c_r(), rc::laser_c_g(), rc::laser_c_b(), 0.0 );
		vis::Draw::add_point( o );

		parts->specific_draw();

		vis::Draw::set_color( rc::laser_c_r(), rc::laser_c_g(), rc::laser_c_b() );
		vis::Draw::add_point( xy );
	vis::Draw::end_line_strip();
}
// process function
void au::Laser::specific_process( int milliseconds ){
	xy +=  v*milliseconds;
	o  += v2*milliseconds;

	parts->specific_process( milliseconds, o, v2 );

	dolitter = checkRing_justdie();
}
// collision functions
void au::Laser::send_collision( int& res, Automaton *a ){
	a->recieve_collision( res, this );
}
