// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef LIGHTNING_HPP
#define LIGHTNING_HPP

#include <map>
#include <list>
#include "Automaton.hpp"
#include "../rc/Constants.hpp"
#include "../util/Random.hpp"

namespace au {

// lightnings fired by the user
class Lightning : public Automaton{
 	public:
		// ctor
		Lightning( const util::Vector& xy, const util::Vector& v, int t = 0,
		           int split = static_cast<int>(util::Random::get(rc::lightning_split_min(),rc::lightning_split_max())),
		           int dead = 0, bool create = true, bool play = true );
		// dtor
		virtual ~Lightning();

		// unserializes the automaton
		static void unserialize( std::istream& in );

		// remove the spark whose dtor is being executed
		void takeout( LightningSpark *me );

	private:
		// processes a branch
		void process_branch( int milliseconds, LightningSpark *ls, std::list<util::Vector>& branch, bool ishead );
		// draws a branch
		void draw_branch( const std::list<util::Vector>& branch, bool ishead );

		// the spark that is taking the lead
		LightningSpark *head;
		std::list<util::Vector> headbranch;
		// the diverged sparks
		std::map< LightningSpark*, std::list<util::Vector> > body; // unordered
		// information of sparks that have died
		std::list< std::list<util::Vector> > pruned;

		// time till next split
		int split;

		// time till dtor once it is already dead
		int dead;

	// redefined automaton functions

	public:
		// serializes the automaton
		virtual void serialize( std::ostream& out );

	protected:
		// drawing function
		virtual void specific_draw();
		// process function
		virtual void specific_process( int milliseconds );
};

}

#endif // Lightning.hpp
