// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include "../rc/Constants.hpp"
#include "../Main.hpp"
#include "LightningSpark.hpp"

// ctor
au::LightningSpark::LightningSpark( Lightning *swarmer, const util::Vector& xy, const util::Vector& v, int lifes, int ttl, int turn, bool bounced ) :
 Automaton( xy, v, rc::lightningspark_r() ),
 lifes( lifes ),
 ttl( ttl ),
 turn( turn ),
 bounced( bounced ),
 swarmer( swarmer )
{
	start_activecollider( SHOT_COL );
	start_passivecollider( EXAMINIS_COL | REDHOLE_COL | SHIELDED_COL );
	start_shot();

	ringdie = true;
}
// dtor
au::LightningSpark::~LightningSpark(){
	swarmer->takeout( this );
}
// gets its velocity
const util::Vector& au::LightningSpark::get_v() const{
	return v;
}
// performs a change in direction
void au::LightningSpark::drift( real newangle ){
	v.angle( newangle );
}
// manages its time-to-live
int au::LightningSpark::get_ttl() const{
	return ttl;
}
void au::LightningSpark::set_ttl( int newttl ){
	ttl = newttl;
}
// manages its turns
int au::LightningSpark::get_turn() const{
	return turn;
}
void au::LightningSpark::set_turn( int newturn ){
	turn = newturn;
}
// gets and clears the bounce
bool au::LightningSpark::has_bounced(){
	bool res = bounced;
	bounced = false;
	return res;
}
// bouncing function
void au::LightningSpark::bounce( const util::Vector& newv ){
	v = newv;
	bounced = true;
}
// each collision type for active colliders
void au::LightningSpark::recieve_collision( int& res, Lagvard *p ){
	Main::add_score( p->get_killscore() );
	res = ( --lifes ) ? PASSIVE_DIES : BOTH_DIE;
}
void au::LightningSpark::recieve_collision( int& res, Proton *p ){
	Main::add_score( p->get_killscore() );
	res = ( --lifes ) ? PASSIVE_DIES : BOTH_DIE;
}
void au::LightningSpark::recieve_collision( int& res, Scared *p ){
	Main::add_score( p->get_killscore() );
	res = ( --lifes ) ? PASSIVE_DIES : BOTH_DIE;
}
void au::LightningSpark::recieve_collision( int& res, Spawned *p ){
	Main::add_score( p->get_killscore() );
	res = ( --lifes ) ? PASSIVE_DIES : BOTH_DIE;
}
void au::LightningSpark::recieve_collision( int& res, Spawnship *p ){
	Main::add_score( p->get_killscore() );
	res = ( --lifes ) ? PASSIVE_DIES : BOTH_DIE;
}
// serializes the automaton
void au::LightningSpark::serialize( std::ostream& out ){
	out << xy << " " << v
	    << " " << lifes << " " << ttl << " " << turn << " " << bounced;
}
// unserializes the automaton
au::LightningSpark *au::LightningSpark::unserialize( std::istream& in, Lightning *swarmer ){
	util::Vector xy;
	util::Vector v;
	int lifes;
	int ttl;
	int turn;
	bool bounced;

	in >> xy >> v
	   >> lifes >> ttl >> turn >> bounced;
	if ( lifes <= 0 )
		throw Main::UNSERIAL_LIGHTNINGSPARK_LIFES;
	if ( ttl <= 0 )
		throw Main::UNSERIAL_LIGHTNINGSPARK_TTL;
	if ( turn <= 0 )
		throw Main::UNSERIAL_LIGHTNINGSPARK_TURN;

	return new LightningSpark( swarmer, xy, v, lifes, ttl, turn, bounced );
}
// collision functions
void au::LightningSpark::send_collision( int& res, Automaton *a ){
	a->recieve_collision( res, this );
}
