// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef LIGHTNINGSPARK_HPP
#define LIGHTNINGSPARK_HPP

#include <list>
#include "Automaton.hpp"

namespace au {

// sparks of a lightning shot
class LightningSpark : public Automaton{
 	public:
		// ctor
		LightningSpark( Lightning *swarmer, const util::Vector& xy, const util::Vector& v, int lifes, int ttl, int turn, bool bounced = false );
		// dtor
		virtual ~LightningSpark();

		// unserializes the automaton
		static LightningSpark *unserialize( std::istream& in, Lightning *swarmer );

		// gets its velocity
		const util::Vector& get_v() const;
		// performs a change in direction
		void drift( real newangle );

		// manages its time-to-live
		int  get_ttl() const;
		void set_ttl( int newttl );

		// manages its turns
		int  get_turn() const;
		void set_turn( int newturn );

		// gets and clears the bounce
		bool has_bounced();

	private:
		// lifes
		int lifes;
		// time-to-live
		int ttl;
		// time till next direction change
		int turn;
		// if it has bounced
		bool bounced;

		// central brain
		Lightning *swarmer;

	// redefined automaton functions

	public:
		// bouncing function
		virtual void bounce( const util::Vector& newv );
		// each collision type for active colliders
		virtual void recieve_collision( int& res, Lagvard *p );
		virtual void recieve_collision( int& res, Proton *p );
		virtual void recieve_collision( int& res, Scared *p );
		virtual void recieve_collision( int& res, Spawned *p );
		virtual void recieve_collision( int& res, Spawnship *p );
		// serializes the automaton
		virtual void serialize( std::ostream& out );

	protected:
		// collision functions
		virtual void send_collision( int& res, Automaton *a );
};

}

#endif // LightningSpark.hpp
