// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include "../rc/Constants.hpp"
#include "../vis/Draw.hpp"
#include "Proton.hpp"

// ctor
au::Proton::Proton( const util::Vector& xy, bool play ) :
 Automaton( xy, util::Vector(0.0,0.0), rc::proton_r() )
{
	start_passivecollider( PLAYER_COL | SHOT_COL );
	start_enemy( play );
	start_processable();
	start_pursuiter();
	start_visible( PROTON_PRIO );
}
// dtor
au::Proton::~Proton(){
	if ( dolitter )
		Automaton::create_debris( xy, rc::proton_litter_r(), rc::proton_litter_c_r(), rc::proton_litter_c_g(), rc::proton_litter_c_b() );
}
// serializes the automaton
void au::Proton::serialize( std::ostream& out ){
	out << Automaton::PROTON_SERIAL
	    << " " << xy << std::endl;
}
// unserializes the automaton
void au::Proton::unserialize( std::istream& in ){
	util::Vector xy;

	in >> xy;

	new Proton( xy, false );
}
// drawing function
void au::Proton::specific_draw(){
	vis::Draw::translate( xy );
	vis::Draw::play_record( vis::PROTON_ID );
}
// where is the nearest player. itis is an unit vector or zero.
// distance is the distance or REAL_MAX
void au::Proton::specific_there( const util::Vector& itis, real distance ){
	v = itis * rc::proton_s();
}
// process function
void au::Proton::specific_process( int milliseconds ){
	xy += v*milliseconds;

	checkRing_stop();
}
// collision functions
void au::Proton::send_collision( int& res, Automaton *a ){
	a->recieve_collision( res, this );
}
// scoring points
int au::Proton::get_killscore() const{
	return rc::proton_kill_score();
}
