// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include "../rc/Constants.hpp"
#include "../vis/Draw.hpp"
#include "../Main.hpp"
#include "Scared.hpp"

// ctor
au::Scared::Scared( const util::Vector& xy, const util::Vector& v, int t, bool play ) :
 Automaton( xy, v, rc::scared_r(), t ),
 f()
{
	m = rc::scared_mass();

	start_passivecollider( PLAYER_COL | SHOT_COL | REDHOLE_COL );
	start_dodger();
	start_enemy( play );
	start_gravitylistener();
	start_processable();
	start_pursuiter();
	start_visible( SCARED_PRIO );
}
// dtor
au::Scared::~Scared(){
	if ( dolitter )
		Automaton::create_debris( xy, rc::scared_litter_r(), rc::scared_litter_c_r(), rc::scared_litter_c_g(), rc::scared_litter_c_b() );
}
// serializes the automaton
void au::Scared::serialize( std::ostream& out ){
	out << Automaton::SCARED_SERIAL
	    << " " << xy << " " << v << " " << t << std::endl;
}
// unserializes the automaton
void au::Scared::unserialize( std::istream& in ){
	util::Vector xy;
	util::Vector v;
	int t;

	in >> xy >> v >> t;
	if ( t < 0 )
		throw Main::UNSERIAL_SCARED;

	new Scared( xy, v, t, false );
}
// drawing function
void au::Scared::specific_draw(){
	vis::Draw::translate( xy );
	if ( t < rc::scared_t() )
		vis::Draw::scale( 1.0+(rc::scared_t()-t)*rc::scared_ds() );
	else
		vis::Draw::rotate( v.angle() ); // look to where it is heading
	vis::Draw::play_record( vis::SCARED_ID );
}
// where is the nearest player. itis is an unit vector or zero.
// distance is the distance or REAL_MAX
void au::Scared::specific_there( const util::Vector& itis, real distance ){
	v = itis * rc::scared_s();
}
// process function
void au::Scared::specific_process( int milliseconds ){
	if ( t > rc::scared_t() ){
		if ( f != util::Vector(0.0, 0.0) )
			v = (v+f).magnitude(rc::scared_s());

		xy += v*milliseconds + g*(milliseconds*rc::non_inertial_time_multiplier());
	}

	checkRing_stop();
}
// dodger parameters
void au::Scared::specific_dodger_parameters( real& fr, real& fi ) const{
	fr = rc::scared_fear_r();
	fi = rc::scared_fear_i();
}
// dodger function
void au::Scared::specific_dodger( const util::Vector& f ){
	this->f = f;
}
// collision functions
void au::Scared::send_collision( int& res, Automaton *a ){
	a->recieve_collision( res, this );
}
// scoring points
int au::Scared::get_killscore() const{
	return rc::scared_kill_score();
}
