// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef SHIELDED_HPP
#define SHIELDED_HPP

#include "Automaton.hpp"
#include "../rc/Constants.hpp"
#include "../util/Random.hpp"

namespace au {

// the shielded enemy
class Shielded : public Automaton{
	public:
		// ctor
		Shielded( const util::Vector& xy, const util::Vector& v = util::Vector(rc::shielded_s()).angle(util::Random::get(PI)),
		          int lifes = rc::shielded_lifes(), real phase = 0.0, bool play = true );
		// dtor
		virtual ~Shielded();

		// unserializes the automaton
		static void unserialize( std::istream& in );

	private:
		// collision checking
		int shot_collision( Automaton *p );

		// lifes
		int lifes;
		// phase angle
		real phase;

	// redefined automaton functions

	public:
		// each collision type for active colliders
		virtual void recieve_collision( int& res, Bullet *p );
		virtual void recieve_collision( int& res, Laser *p );
		virtual void recieve_collision( int& res, LightningSpark *p );
		virtual void recieve_collision( int& res, Shielded *p );
		// scoring points
		virtual int get_killscore() const;
		virtual int get_hitscore() const;
		// serializes the automaton
		virtual void serialize( std::ostream& out );

	protected:
		// drawing function
		virtual void specific_draw();
		// process function
		virtual void specific_process( int milliseconds );
		// collision functions
		virtual void send_collision( int& res, Automaton *a );
};

}

#endif // Shielded.hpp
