// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#include <cmath>
#include <algorithm>
#include <boost/foreach.hpp>
#define foreach BOOST_FOREACH
#include "../rc/Constants.hpp"
#include "../vis/Draw.hpp"
#include "../Main.hpp"
#include "Spawn.hpp"

// ctor
au::Spawn::Spawn( const util::Vector& xy, real r ) :
 Automaton( xy, util::Vector(0.0,0.0), r ),
 d( REAL_MAX ),
 children()
{
	start_processable();
	start_pursuiter();
	start_visible( SPAWNED_PRIO );
}
// dtor
au::Spawn::~Spawn(){
}
// adds the spawned
void au::Spawn::add_child( Spawned *me ){
	children.push_back( me );
}
// remove the spawned whose dtor is being executed
void au::Spawn::takeout( Spawned *me ){
	// deleting
	children.remove( me );

	// recalculating center
	if ( children.size() == 2 ){
		xy = (children.back()->get_xy() + children.front()->get_xy())/2.0;

		// recalculating phases
		children.front()->set_phase( (children.front()->get_xy() - xy).angle() );
		children.back()->set_phase( (children.back()->get_xy() - xy).angle() );
	}else if ( children.size() == 1 ){
		xy = children.front()->get_xy();
	}else
		delete this;
}
// serializes the automaton
void au::Spawn::serialize( std::ostream& out ){
	out << Automaton::SPAWN_SERIAL
	    << " " << xy << " " << r
	    << " " << children.size();
	foreach( Spawned *sd, children )
		out << " " << sd->get_xy() << " " << sd->get_phase();
	out << std::endl;
}
// unserializes the automaton
void au::Spawn::unserialize( std::istream& in ){
	util::Vector xy;
	real r;
	int cs;

	in >> xy >> r
	   >> cs;
	if ( (r < 0) || (r > rc::spawn_r()) )
		throw Main::UNSERIAL_SPAWN_R;
	if ( (cs <= 0) || (cs > 3) )
		throw Main::UNSERIAL_SPAWN_CHILDREN;

	Spawn *s = new Spawn( xy, r );

	for( ; cs ; --cs ){
		real phase;

		in >> xy >> phase;

		s->add_child( new Spawned( s, xy, phase, false ) );
	}
}
// drawing function
void au::Spawn::specific_draw(){
	foreach( Spawned *sd, children ){
		vis::Draw::reset();
		vis::Draw::translate( sd->get_xy() );
		vis::Draw::play_record( vis::SPAWNED_ID );
	}
}
// where is the nearest player. itis is an unit vector or zero.
// distance is the distance or REAL_MAX
void au::Spawn::specific_there( const util::Vector& itis, real distance ){
	v = itis * rc::spawn_s();
	d = distance;
}
// process function
void au::Spawn::specific_process( int milliseconds ){
	if ( d < r ){
		// if player is really close, collapse the spawneds
		r = std::max( static_cast<real>(0.0), r - rc::spawn_r()*(static_cast<real>(milliseconds)/rc::spawn_t()) );
	}else if ( r < rc::spawn_r() ){
		// else set them in a circle
		r = std::min( rc::spawn_r(), r + rc::spawn_r()*(static_cast<real>(milliseconds)/rc::spawn_t()) );
	}

	xy += v*milliseconds;

	// move the spawneds
	if ( children.size() == 1 )
		children.front()->set_xy( xy );
	else{
		foreach( Spawned *sd, children ){
			sd->set_phase( std::fmod( sd->get_phase()+(rc::spawn_as()*milliseconds), static_cast<real>(2.0*PI) ) );
			sd->set_xy( xy + util::Vector( r ).angle( sd->get_phase() ) );
		}
	}
}
