// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef SPAWN_HPP
#define SPAWN_HPP

#include <list>
#include "Automaton.hpp"

namespace au {

// the class that controls the spawned enemies
class Spawn : public Automaton{
 	public:
		// ctor
		Spawn( const util::Vector& xy, real r = 0.0 );
		// dtor
		virtual ~Spawn();

		// unserializes the automaton
		static void unserialize( std::istream& in );

		// adds the spawned
		void add_child( Spawned *me );

		// remove the spawned whose dtor is being executed
		void takeout( Spawned *me );

	private:
		// distance form player
		real d;
		// spawneds
		std::list<Spawned*> children;

	// redefined automaton functions

	public:
		// serializes the automaton
		virtual void serialize( std::ostream& out );

	protected:
		// drawing function
		virtual void specific_draw();
		// where is the nearest player. itis is an unit vector or zero.
		// distance is the distance or REAL_MAX
		virtual void specific_there( const util::Vector& itis, real distance );
		// process function
		virtual void specific_process( int milliseconds );
};

}

#endif // Spawn.hpp
