// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef SPRING_HPP
#define SPRING_HPP

#include "Automaton.hpp"

namespace au {

// a spring fixed at its origin and moved by gravity
class Spring : public Automaton{
	public:
		// ctor
		Spring( const util::Vector& xy, const util::Vector& v = util::Vector(0.0,0.0), real alpha = 0.0 );
		// dtor
		virtual ~Spring();

		// gets alpha color component
		real get_color_alpha() const;

		// postoperations when grid serialization has finished
		void end_serialize();
		// unserializes the automaton
		static Spring *unserialize( std::istream& in );

	private:
		// spring's original position
		util::Vector o;
		// alpha component
		real alpha;

	// redefined automaton functions

	public:
		// serializes the automaton
		virtual void serialize( std::ostream& out );

	protected:
		// process function
		virtual void specific_process( int milliseconds );
};

}

#endif // Spring.hpp
