// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef GUI_HPP
#define GUI_HPP

// Class that prints the stats info onto the screen.
// It has the base format:
//   "SCORE: 1234567   ###   BOMBS: 1   HISCORE: 1234567"

#include <vector>
#include <string>
#include "../vis/Font.hpp"

namespace gui {

class GUI{
	public:
		// initializes the gui [depends on: Locale]
		static void init();
		// ends the gui
		static void finalize();

		// drawing function
		static void draw();
		// initialize drawing
		static void initdrawing();

		// updating GUI functions
		static void update();
		static void updateScore();
		static void updateGameover();
		static void updateLifes();
		static void updateBombs();
		static void updateHiscore();

	private:
		// gui text
		static std::vector<std::string> statetext;
		static std::vector<std::string> gameovertext;

		// information has been updated
		static bool updated;

		// color callbacks
		static void statecolor( int part );
		static void gameovercolor( int part );

		// what will be printed to screen
		static vis::Font *state;
		static vis::Font *gameover;
};

}

#endif // GUI.hpp
