// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef MENU_HPP
#define MENU_HPP

#include <SDL.h>
#include "../vis/Font.hpp"
#include "Menu_Component.hpp"

namespace gui {

class Menu{
	public:
		// initializes the menu [depends on: -]
		static void init();
		// ends the menu
		static void finalize();

		// drawing function
		static void draw();
		// initialize drawing
		static void initdrawing();

		// processing function
		static void process( int milliseconds );

		// event capturing functions
		static void captureKey( SDLKey key );
		static void captureButton( int which, int button );
		static void captureAxis( int which, int axis, int value );

		// menu has been activated?
		static bool is_on();

		// interface accessed only from panes:

		// updates the menu
		static void update();
		// starts the who pane_id pane or turns off the menu if who is NO_PANE
		static void start( int who );
		// starts a capture_t type capture and will notify pane notifywho
		static void capture( int type, int notifywho );

		// the various menu panes
		enum pane_id {
			ROOT_PANE = 0,
			CONFIG_PANE,
			CONTROLS_PANE,

			NO_PANE
		};
		// type of the event capture
		enum capture_t {
			NO_CAPTURE = 0,
			CAPTURE_KEY,
			CAPTURE_JOYSTICK_BUTTON,
			CAPTURE_JOYSTICK_AXIS
		};

	private:
		// color callback
		static void color( int part );

		// turn on/off the menu
		static void turnon();
		static void turnoff();

		// menu has been activated
		static bool on;

		// time since last player interaction
		static int time;

		// menu drawing has to be updated
		static bool updated;

		// active pane
		static int pane;
		// available panes
		static Menu_Component *panes[];
		// pane's renders
		static vis::Font *renders[];

		// capturing?
		static bool capturing;
		// present capturing process type
		static int  capture_type;
		// pane to notify
		static int  capture_notifywho;
		// active joystick axis capture data
		static int  capture_jwhich;
		static int  capture_jaxis;
		static int  capture_jvalue;
		static bool capture_jcount;
		static int  capture_jtime;
};

}

#endif // Menu.hpp
