// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef MENU_COMPONENT_HPP
#define MENU_COMPONENT_HPP

#include <vector>
#include <string>
#include <SDL.h>

namespace gui {

class Menu_Component{
	public:
		// ctor
		Menu_Component();
		// dtor
		virtual ~Menu_Component();

		// gets pane's active line
		int get_line() const;
		// gets the menu text
		const std::vector<std::string>& get_text() const;

		// pane start
		virtual void start( int from ) = 0;
		// functions to process user events
		virtual void exit() = 0;
		virtual void move( int updown ) = 0; // -1: up, 1: down
		virtual void change( int moreless ) = 0; // -1: less, 1: more
		virtual void accept() = 0;

		// capture ended
		virtual void notify( bool success, SDLKey key, int joy, int axis_but, bool invert ) = 0;

	protected:
		// pane's active line
		int line;
		// menu text
		std::vector<std::string> text;
};

}

#endif // Menu_Component.hpp
