// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef MATRIX_HPP
#define MATRIX_HPP

namespace util {

template<typename T>
class Matrix{
	public:
		// empty ctor
		Matrix() :
		 rows(0),
		 cols(0)
		{
			matrix = NULL;
		}

		// ctor for a M[rows][cols] matrix
		Matrix( int rows, int cols ) :
		 rows(rows),
		 cols(cols)
		{
			matrix = new T[rows*cols];
		}

		// ctor for a M[rows][cols] matrix initialized with value
		Matrix( int rows, int cols, T value ) :
		 rows(rows),
		 cols(cols)
		{
			matrix = new T[rows*cols];

			for( int i=0 ; i<rows*cols ; ++i )
				matrix[i] = value;
		}

		// dtor, does not free elements stored in the matrix
		virtual ~Matrix(){
			if ( matrix != NULL )
				delete[] matrix;
		}

		// assigns a matrix
		Matrix& operator=( const Matrix& mtrx ){
			if ( &mtrx == this )
				return *this;

			if ( matrix != NULL )
				delete[] matrix;

			if ( mtrx.empty() ){
				rows = 0;
				cols = 0;
				matrix = NULL;
			}else{
				rows = mtrx.rows;
				cols = mtrx.cols;

				matrix = new T[rows*cols];

				for( int i=0 ; i<rows*cols ; ++i )
					matrix[i] = mtrx.matrix[i];
			}

			return *this;
		}

		// accesses a row
		T* operator[]( int row ) const{
			return matrix + row*cols;
		}

		// checks if matrix has no positions
		bool empty() const{
			return matrix == NULL;
		}

		// sets the matrix as an empty matrix
		void clear(){
			if ( matrix != NULL )
				delete[] matrix;

			rows = 0;
			cols = 0;
			matrix = NULL;
		}

		// getters
		int get_rows() const{
			return rows;
		}
		int get_cols() const{
			return cols;
		}

	private:
		// data members
		int rows;
		int cols;

		T *matrix;
};

}

#endif // Matrix.hpp
