// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef SCREEN_HPP
#define SCREEN_HPP

#include <set>
#include <utility>
#include <SDL.h>
#include "../util/Real.hpp"

namespace vis {

class Screen{
	public:
		// (re)initializes the screen [depends on: Config, GUI]
		static void init();

		// prepares the screen for drawing
		static void start_drawing();
		// all drawing has been done
		static void end_drawing();
		// saves a screenshot
		static void takescreenshot();

		// available resolutions
		// always at least one: the present resolution
		static const std::set< std::pair<int,int> >& get_availableres();

		// gets present pixelsize, i.e. pixels per rc::width()
		static real get_pixelsize();

	private:
		// available resolutions
		// always at least one: the present resolution
		static std::set< std::pair<int,int> > available;

		// pixel size in game's dimensions
		static real pixelsize;

		// screen's SDL surface
		static SDL_Surface *surf;
};

}

#endif // Screen.hpp
