// nvsts is a multi-directional shooter game.
// Copyright (C) 2008 Carles Creus Lopez
//
// nvsts is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>
// or write to the Free Software Foundation, Inc., 51 Franklin Street,
// Fifth Floor, Boston, MA 02110-1301 USA.
//
// You may contact the author at <crazycraft@gmail.com>

#ifndef BLAST_HPP
#define BLAST_HPP

#include "Automaton.hpp"

namespace au {

// another game-useless class, but also looks nice
class Blast : public Automaton{
	public:
		// ctor
		Blast( const util::Vector& xy, real r = 0.0, int t = 0, bool active = true );
		// dtor
		virtual ~Blast();

		// unserializes the automaton
		static void unserialize( std::istream& in );

	private:
		// debris moved by the blast
		typedef struct {
			// position
			util::Vector xy;
			// speed
			util::Vector v;
			// color
			real red;
			real green;
			real blue;
			real alpha;
		} rubble_type;
		std::list<rubble_type> rub;
		// is the blast active?
		bool active;

	// redefined automaton functions

	public:
		// each collision type for active colliders
		virtual void recieve_collision( int& res, Debris *p );
		// serializes the automaton
		virtual void serialize( std::ostream& out );

	protected:
		// drawing function
		virtual void specific_draw();
		// process function
		virtual void specific_process( int milliseconds );
};

}

#endif // Blast.hpp
